# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("powrprof.dll")
prototypes = \
    {
        # 
        'CallNtPowerInformation': SimTypeFunction([SimTypeInt(signed=False, label="POWER_INFORMATION_LEVEL"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["InformationLevel", "InputBuffer", "InputBufferLength", "OutputBuffer", "OutputBufferLength"]),
        # 
        'GetPwrCapabilities': SimTypeFunction([SimTypePointer(SimStruct({"PowerButtonPresent": SimTypeChar(label="Byte"), "SleepButtonPresent": SimTypeChar(label="Byte"), "LidPresent": SimTypeChar(label="Byte"), "SystemS1": SimTypeChar(label="Byte"), "SystemS2": SimTypeChar(label="Byte"), "SystemS3": SimTypeChar(label="Byte"), "SystemS4": SimTypeChar(label="Byte"), "SystemS5": SimTypeChar(label="Byte"), "HiberFilePresent": SimTypeChar(label="Byte"), "FullWake": SimTypeChar(label="Byte"), "VideoDimPresent": SimTypeChar(label="Byte"), "ApmPresent": SimTypeChar(label="Byte"), "UpsPresent": SimTypeChar(label="Byte"), "ThermalControl": SimTypeChar(label="Byte"), "ProcessorThrottle": SimTypeChar(label="Byte"), "ProcessorMinThrottle": SimTypeChar(label="Byte"), "ProcessorMaxThrottle": SimTypeChar(label="Byte"), "FastSystemS4": SimTypeChar(label="Byte"), "Hiberboot": SimTypeChar(label="Byte"), "WakeAlarmPresent": SimTypeChar(label="Byte"), "AoAc": SimTypeChar(label="Byte"), "DiskSpinDown": SimTypeChar(label="Byte"), "HiberFileType": SimTypeChar(label="Byte"), "AoAcConnectivitySupported": SimTypeChar(label="Byte"), "spare3": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6), "SystemBatteriesPresent": SimTypeChar(label="Byte"), "BatteriesAreShortTerm": SimTypeChar(label="Byte"), "BatteryScale": SimTypeFixedSizeArray(SimStruct({"Granularity": SimTypeInt(signed=False, label="UInt32"), "Capacity": SimTypeInt(signed=False, label="UInt32")}, name="BATTERY_REPORTING_SCALE", pack=False, align=None), 3), "AcOnLineWake": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "SoftLidWake": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "RtcWake": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "MinDeviceWakeState": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "DefaultLowLatencyWake": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")}, name="SYSTEM_POWER_CAPABILITIES", pack=False, align=None), offset=0)], SimTypeChar(label="Byte"), arg_names=["lpspc"]),
        # 
        'PowerDeterminePlatformRoleEx': SimTypeFunction([SimTypeInt(signed=False, label="POWER_PLATFORM_ROLE_VERSION")], SimTypeInt(signed=False, label="POWER_PLATFORM_ROLE"), arg_names=["Version"]),
        # 
        'PowerRegisterSuspendResumeNotification': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Flags", "Recipient", "RegistrationHandle"]),
        # 
        'PowerUnregisterSuspendResumeNotification': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RegistrationHandle"]),
        # 
        'PowerReadACValue': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SchemeGuid", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "Type", "Buffer", "BufferSize"]),
        # 
        'PowerReadDCValue': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SchemeGuid", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "Type", "Buffer", "BufferSize"]),
        # 
        'PowerWriteACValueIndex': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SchemeGuid", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "AcValueIndex"]),
        # 
        'PowerWriteDCValueIndex': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SchemeGuid", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "DcValueIndex"]),
        # 
        'PowerGetActiveScheme': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Guid"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["UserRootPowerKey", "ActivePolicyGuid"]),
        # 
        'PowerSetActiveScheme': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["UserRootPowerKey", "SchemeGuid"]),
        # 
        'PowerSettingRegisterNotification': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="POWER_SETTING_REGISTER_NOTIFICATION_FLAGS"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["SettingGuid", "Flags", "Recipient", "RegistrationHandle"]),
        # 
        'PowerSettingUnregisterNotification': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RegistrationHandle"]),
        # 
        'PowerRegisterForEffectivePowerModeNotifications': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="EFFECTIVE_POWER_MODE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Mode", "Context"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Version", "Callback", "Context", "RegistrationHandle"]),
        # 
        'PowerUnregisterFromEffectivePowerModeNotifications': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RegistrationHandle"]),
        # 
        'GetPwrDiskSpindownRange': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeChar(label="Byte"), arg_names=["puiMax", "puiMin"]),
        # 
        'EnumPwrSchemes': SimTypeFunction([SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"user": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "IdleAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "IdleDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "IdleTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "IdleTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "IdleSensitivityAc": SimTypeChar(label="Byte"), "IdleSensitivityDc": SimTypeChar(label="Byte"), "ThrottlePolicyAc": SimTypeChar(label="Byte"), "ThrottlePolicyDc": SimTypeChar(label="Byte"), "MaxSleepAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "MaxSleepDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "Reserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 2), "VideoTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "VideoTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "SpindownTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "SpindownTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "OptimizeForPowerAc": SimTypeChar(label="Byte"), "OptimizeForPowerDc": SimTypeChar(label="Byte"), "FanThrottleToleranceAc": SimTypeChar(label="Byte"), "FanThrottleToleranceDc": SimTypeChar(label="Byte"), "ForcedThrottleAc": SimTypeChar(label="Byte"), "ForcedThrottleDc": SimTypeChar(label="Byte")}, name="USER_POWER_POLICY", pack=False, align=None), "mach": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "MinSleepAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "MinSleepDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "ReducedLatencySleepAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "ReducedLatencySleepDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "DozeTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "DozeTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "DozeS4TimeoutAc": SimTypeInt(signed=False, label="UInt32"), "DozeS4TimeoutDc": SimTypeInt(signed=False, label="UInt32"), "MinThrottleAc": SimTypeChar(label="Byte"), "MinThrottleDc": SimTypeChar(label="Byte"), "pad1": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 2), "OverThrottledAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "OverThrottledDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None)}, name="MACHINE_POWER_POLICY", pack=False, align=None)}, name="POWER_POLICY", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeChar(label="Byte"), arg_names=["Index", "NameSize", "Name", "DescriptionSize", "Description", "Policy", "Context"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeChar(label="Byte"), arg_names=["lpfn", "lParam"]),
        # 
        'ReadGlobalPwrPolicy': SimTypeFunction([SimTypePointer(SimStruct({"user": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "PowerButtonAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "PowerButtonDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "SleepButtonAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "SleepButtonDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "LidCloseAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "LidCloseDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "DischargePolicy": SimTypeFixedSizeArray(SimStruct({"Enable": SimTypeChar(label="Byte"), "Spare": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 3), "BatteryLevel": SimTypeInt(signed=False, label="UInt32"), "PowerPolicy": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "MinSystemState": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")}, name="SYSTEM_POWER_LEVEL", pack=False, align=None), 4), "GlobalFlags": SimTypeInt(signed=False, label="UInt32")}, name="GLOBAL_USER_POWER_POLICY", pack=False, align=None), "mach": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "LidOpenWakeAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "LidOpenWakeDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "BroadcastCapacityResolution": SimTypeInt(signed=False, label="UInt32")}, name="GLOBAL_MACHINE_POWER_POLICY", pack=False, align=None)}, name="GLOBAL_POWER_POLICY", pack=False, align=None), offset=0)], SimTypeChar(label="Byte"), arg_names=["pGlobalPowerPolicy"]),
        # 
        'ReadPwrScheme': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"user": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "IdleAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "IdleDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "IdleTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "IdleTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "IdleSensitivityAc": SimTypeChar(label="Byte"), "IdleSensitivityDc": SimTypeChar(label="Byte"), "ThrottlePolicyAc": SimTypeChar(label="Byte"), "ThrottlePolicyDc": SimTypeChar(label="Byte"), "MaxSleepAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "MaxSleepDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "Reserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 2), "VideoTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "VideoTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "SpindownTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "SpindownTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "OptimizeForPowerAc": SimTypeChar(label="Byte"), "OptimizeForPowerDc": SimTypeChar(label="Byte"), "FanThrottleToleranceAc": SimTypeChar(label="Byte"), "FanThrottleToleranceDc": SimTypeChar(label="Byte"), "ForcedThrottleAc": SimTypeChar(label="Byte"), "ForcedThrottleDc": SimTypeChar(label="Byte")}, name="USER_POWER_POLICY", pack=False, align=None), "mach": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "MinSleepAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "MinSleepDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "ReducedLatencySleepAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "ReducedLatencySleepDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "DozeTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "DozeTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "DozeS4TimeoutAc": SimTypeInt(signed=False, label="UInt32"), "DozeS4TimeoutDc": SimTypeInt(signed=False, label="UInt32"), "MinThrottleAc": SimTypeChar(label="Byte"), "MinThrottleDc": SimTypeChar(label="Byte"), "pad1": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 2), "OverThrottledAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "OverThrottledDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None)}, name="MACHINE_POWER_POLICY", pack=False, align=None)}, name="POWER_POLICY", pack=False, align=None), offset=0)], SimTypeChar(label="Byte"), arg_names=["uiID", "pPowerPolicy"]),
        # 
        'WritePwrScheme': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"user": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "IdleAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "IdleDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "IdleTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "IdleTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "IdleSensitivityAc": SimTypeChar(label="Byte"), "IdleSensitivityDc": SimTypeChar(label="Byte"), "ThrottlePolicyAc": SimTypeChar(label="Byte"), "ThrottlePolicyDc": SimTypeChar(label="Byte"), "MaxSleepAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "MaxSleepDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "Reserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 2), "VideoTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "VideoTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "SpindownTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "SpindownTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "OptimizeForPowerAc": SimTypeChar(label="Byte"), "OptimizeForPowerDc": SimTypeChar(label="Byte"), "FanThrottleToleranceAc": SimTypeChar(label="Byte"), "FanThrottleToleranceDc": SimTypeChar(label="Byte"), "ForcedThrottleAc": SimTypeChar(label="Byte"), "ForcedThrottleDc": SimTypeChar(label="Byte")}, name="USER_POWER_POLICY", pack=False, align=None), "mach": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "MinSleepAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "MinSleepDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "ReducedLatencySleepAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "ReducedLatencySleepDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "DozeTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "DozeTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "DozeS4TimeoutAc": SimTypeInt(signed=False, label="UInt32"), "DozeS4TimeoutDc": SimTypeInt(signed=False, label="UInt32"), "MinThrottleAc": SimTypeChar(label="Byte"), "MinThrottleDc": SimTypeChar(label="Byte"), "pad1": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 2), "OverThrottledAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "OverThrottledDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None)}, name="MACHINE_POWER_POLICY", pack=False, align=None)}, name="POWER_POLICY", pack=False, align=None), offset=0)], SimTypeChar(label="Byte"), arg_names=["puiID", "lpszSchemeName", "lpszDescription", "lpScheme"]),
        # 
        'WriteGlobalPwrPolicy': SimTypeFunction([SimTypePointer(SimStruct({"user": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "PowerButtonAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "PowerButtonDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "SleepButtonAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "SleepButtonDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "LidCloseAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "LidCloseDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "DischargePolicy": SimTypeFixedSizeArray(SimStruct({"Enable": SimTypeChar(label="Byte"), "Spare": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 3), "BatteryLevel": SimTypeInt(signed=False, label="UInt32"), "PowerPolicy": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "MinSystemState": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")}, name="SYSTEM_POWER_LEVEL", pack=False, align=None), 4), "GlobalFlags": SimTypeInt(signed=False, label="UInt32")}, name="GLOBAL_USER_POWER_POLICY", pack=False, align=None), "mach": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "LidOpenWakeAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "LidOpenWakeDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "BroadcastCapacityResolution": SimTypeInt(signed=False, label="UInt32")}, name="GLOBAL_MACHINE_POWER_POLICY", pack=False, align=None)}, name="GLOBAL_POWER_POLICY", pack=False, align=None), offset=0)], SimTypeChar(label="Byte"), arg_names=["pGlobalPowerPolicy"]),
        # 
        'DeletePwrScheme': SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["uiID"]),
        # 
        'GetActivePwrScheme': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeChar(label="Byte"), arg_names=["puiID"]),
        # 
        'SetActivePwrScheme': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"user": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "PowerButtonAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "PowerButtonDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "SleepButtonAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "SleepButtonDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "LidCloseAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "LidCloseDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "DischargePolicy": SimTypeFixedSizeArray(SimStruct({"Enable": SimTypeChar(label="Byte"), "Spare": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 3), "BatteryLevel": SimTypeInt(signed=False, label="UInt32"), "PowerPolicy": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "MinSystemState": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")}, name="SYSTEM_POWER_LEVEL", pack=False, align=None), 4), "GlobalFlags": SimTypeInt(signed=False, label="UInt32")}, name="GLOBAL_USER_POWER_POLICY", pack=False, align=None), "mach": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "LidOpenWakeAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "LidOpenWakeDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "BroadcastCapacityResolution": SimTypeInt(signed=False, label="UInt32")}, name="GLOBAL_MACHINE_POWER_POLICY", pack=False, align=None)}, name="GLOBAL_POWER_POLICY", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"user": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "IdleAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "IdleDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "IdleTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "IdleTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "IdleSensitivityAc": SimTypeChar(label="Byte"), "IdleSensitivityDc": SimTypeChar(label="Byte"), "ThrottlePolicyAc": SimTypeChar(label="Byte"), "ThrottlePolicyDc": SimTypeChar(label="Byte"), "MaxSleepAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "MaxSleepDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "Reserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 2), "VideoTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "VideoTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "SpindownTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "SpindownTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "OptimizeForPowerAc": SimTypeChar(label="Byte"), "OptimizeForPowerDc": SimTypeChar(label="Byte"), "FanThrottleToleranceAc": SimTypeChar(label="Byte"), "FanThrottleToleranceDc": SimTypeChar(label="Byte"), "ForcedThrottleAc": SimTypeChar(label="Byte"), "ForcedThrottleDc": SimTypeChar(label="Byte")}, name="USER_POWER_POLICY", pack=False, align=None), "mach": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "MinSleepAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "MinSleepDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "ReducedLatencySleepAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "ReducedLatencySleepDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "DozeTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "DozeTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "DozeS4TimeoutAc": SimTypeInt(signed=False, label="UInt32"), "DozeS4TimeoutDc": SimTypeInt(signed=False, label="UInt32"), "MinThrottleAc": SimTypeChar(label="Byte"), "MinThrottleDc": SimTypeChar(label="Byte"), "pad1": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 2), "OverThrottledAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "OverThrottledDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None)}, name="MACHINE_POWER_POLICY", pack=False, align=None)}, name="POWER_POLICY", pack=False, align=None), offset=0)], SimTypeChar(label="Byte"), arg_names=["uiID", "pGlobalPowerPolicy", "pPowerPolicy"]),
        # 
        'IsPwrSuspendAllowed': SimTypeFunction([], SimTypeChar(label="Byte")),
        # 
        'IsPwrHibernateAllowed': SimTypeFunction([], SimTypeChar(label="Byte")),
        # 
        'IsPwrShutdownAllowed': SimTypeFunction([], SimTypeChar(label="Byte")),
        # 
        'IsAdminOverrideActive': SimTypeFunction([SimTypePointer(SimStruct({"MinSleep": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "MaxSleep": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "MinVideoTimeout": SimTypeInt(signed=False, label="UInt32"), "MaxVideoTimeout": SimTypeInt(signed=False, label="UInt32"), "MinSpindownTimeout": SimTypeInt(signed=False, label="UInt32"), "MaxSpindownTimeout": SimTypeInt(signed=False, label="UInt32")}, name="ADMINISTRATOR_POWER_POLICY", pack=False, align=None), offset=0)], SimTypeChar(label="Byte"), arg_names=["papp"]),
        # 
        'SetSuspendState': SimTypeFunction([SimTypeChar(label="Byte"), SimTypeChar(label="Byte"), SimTypeChar(label="Byte")], SimTypeChar(label="Byte"), arg_names=["bHibernate", "bForce", "bWakeupEventsDisabled"]),
        # 
        'GetCurrentPowerPolicies': SimTypeFunction([SimTypePointer(SimStruct({"user": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "PowerButtonAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "PowerButtonDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "SleepButtonAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "SleepButtonDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "LidCloseAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "LidCloseDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "DischargePolicy": SimTypeFixedSizeArray(SimStruct({"Enable": SimTypeChar(label="Byte"), "Spare": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 3), "BatteryLevel": SimTypeInt(signed=False, label="UInt32"), "PowerPolicy": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "MinSystemState": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")}, name="SYSTEM_POWER_LEVEL", pack=False, align=None), 4), "GlobalFlags": SimTypeInt(signed=False, label="UInt32")}, name="GLOBAL_USER_POWER_POLICY", pack=False, align=None), "mach": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "LidOpenWakeAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "LidOpenWakeDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "BroadcastCapacityResolution": SimTypeInt(signed=False, label="UInt32")}, name="GLOBAL_MACHINE_POWER_POLICY", pack=False, align=None)}, name="GLOBAL_POWER_POLICY", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"user": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "IdleAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "IdleDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "IdleTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "IdleTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "IdleSensitivityAc": SimTypeChar(label="Byte"), "IdleSensitivityDc": SimTypeChar(label="Byte"), "ThrottlePolicyAc": SimTypeChar(label="Byte"), "ThrottlePolicyDc": SimTypeChar(label="Byte"), "MaxSleepAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "MaxSleepDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "Reserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 2), "VideoTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "VideoTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "SpindownTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "SpindownTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "OptimizeForPowerAc": SimTypeChar(label="Byte"), "OptimizeForPowerDc": SimTypeChar(label="Byte"), "FanThrottleToleranceAc": SimTypeChar(label="Byte"), "FanThrottleToleranceDc": SimTypeChar(label="Byte"), "ForcedThrottleAc": SimTypeChar(label="Byte"), "ForcedThrottleDc": SimTypeChar(label="Byte")}, name="USER_POWER_POLICY", pack=False, align=None), "mach": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "MinSleepAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "MinSleepDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "ReducedLatencySleepAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "ReducedLatencySleepDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "DozeTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "DozeTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "DozeS4TimeoutAc": SimTypeInt(signed=False, label="UInt32"), "DozeS4TimeoutDc": SimTypeInt(signed=False, label="UInt32"), "MinThrottleAc": SimTypeChar(label="Byte"), "MinThrottleDc": SimTypeChar(label="Byte"), "pad1": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 2), "OverThrottledAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "OverThrottledDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None)}, name="MACHINE_POWER_POLICY", pack=False, align=None)}, name="POWER_POLICY", pack=False, align=None), offset=0)], SimTypeChar(label="Byte"), arg_names=["pGlobalPowerPolicy", "pPowerPolicy"]),
        # 
        'CanUserWritePwrScheme': SimTypeFunction([], SimTypeChar(label="Byte")),
        # 
        'ReadProcessorPwrScheme': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "ProcessorPolicyAc": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "DynamicThrottle": SimTypeChar(label="Byte"), "Spare": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 3), "_bitfield": SimTypeInt(signed=False, label="UInt32"), "PolicyCount": SimTypeInt(signed=False, label="UInt32"), "Policy": SimTypeFixedSizeArray(SimStruct({"TimeCheck": SimTypeInt(signed=False, label="UInt32"), "DemoteLimit": SimTypeInt(signed=False, label="UInt32"), "PromoteLimit": SimTypeInt(signed=False, label="UInt32"), "DemotePercent": SimTypeChar(label="Byte"), "PromotePercent": SimTypeChar(label="Byte"), "Spare": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 2), "_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="PROCESSOR_POWER_POLICY_INFO", pack=False, align=None), 3)}, name="PROCESSOR_POWER_POLICY", pack=False, align=None), "ProcessorPolicyDc": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "DynamicThrottle": SimTypeChar(label="Byte"), "Spare": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 3), "_bitfield": SimTypeInt(signed=False, label="UInt32"), "PolicyCount": SimTypeInt(signed=False, label="UInt32"), "Policy": SimTypeFixedSizeArray(SimStruct({"TimeCheck": SimTypeInt(signed=False, label="UInt32"), "DemoteLimit": SimTypeInt(signed=False, label="UInt32"), "PromoteLimit": SimTypeInt(signed=False, label="UInt32"), "DemotePercent": SimTypeChar(label="Byte"), "PromotePercent": SimTypeChar(label="Byte"), "Spare": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 2), "_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="PROCESSOR_POWER_POLICY_INFO", pack=False, align=None), 3)}, name="PROCESSOR_POWER_POLICY", pack=False, align=None)}, name="MACHINE_PROCESSOR_POWER_POLICY", pack=False, align=None), offset=0)], SimTypeChar(label="Byte"), arg_names=["uiID", "pMachineProcessorPowerPolicy"]),
        # 
        'WriteProcessorPwrScheme': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "ProcessorPolicyAc": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "DynamicThrottle": SimTypeChar(label="Byte"), "Spare": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 3), "_bitfield": SimTypeInt(signed=False, label="UInt32"), "PolicyCount": SimTypeInt(signed=False, label="UInt32"), "Policy": SimTypeFixedSizeArray(SimStruct({"TimeCheck": SimTypeInt(signed=False, label="UInt32"), "DemoteLimit": SimTypeInt(signed=False, label="UInt32"), "PromoteLimit": SimTypeInt(signed=False, label="UInt32"), "DemotePercent": SimTypeChar(label="Byte"), "PromotePercent": SimTypeChar(label="Byte"), "Spare": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 2), "_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="PROCESSOR_POWER_POLICY_INFO", pack=False, align=None), 3)}, name="PROCESSOR_POWER_POLICY", pack=False, align=None), "ProcessorPolicyDc": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "DynamicThrottle": SimTypeChar(label="Byte"), "Spare": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 3), "_bitfield": SimTypeInt(signed=False, label="UInt32"), "PolicyCount": SimTypeInt(signed=False, label="UInt32"), "Policy": SimTypeFixedSizeArray(SimStruct({"TimeCheck": SimTypeInt(signed=False, label="UInt32"), "DemoteLimit": SimTypeInt(signed=False, label="UInt32"), "PromoteLimit": SimTypeInt(signed=False, label="UInt32"), "DemotePercent": SimTypeChar(label="Byte"), "PromotePercent": SimTypeChar(label="Byte"), "Spare": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 2), "_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="PROCESSOR_POWER_POLICY_INFO", pack=False, align=None), 3)}, name="PROCESSOR_POWER_POLICY", pack=False, align=None)}, name="MACHINE_PROCESSOR_POWER_POLICY", pack=False, align=None), offset=0)], SimTypeChar(label="Byte"), arg_names=["uiID", "pMachineProcessorPowerPolicy"]),
        # 
        'ValidatePowerPolicies': SimTypeFunction([SimTypePointer(SimStruct({"user": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "PowerButtonAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "PowerButtonDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "SleepButtonAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "SleepButtonDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "LidCloseAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "LidCloseDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "DischargePolicy": SimTypeFixedSizeArray(SimStruct({"Enable": SimTypeChar(label="Byte"), "Spare": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 3), "BatteryLevel": SimTypeInt(signed=False, label="UInt32"), "PowerPolicy": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "MinSystemState": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE")}, name="SYSTEM_POWER_LEVEL", pack=False, align=None), 4), "GlobalFlags": SimTypeInt(signed=False, label="UInt32")}, name="GLOBAL_USER_POWER_POLICY", pack=False, align=None), "mach": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "LidOpenWakeAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "LidOpenWakeDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "BroadcastCapacityResolution": SimTypeInt(signed=False, label="UInt32")}, name="GLOBAL_MACHINE_POWER_POLICY", pack=False, align=None)}, name="GLOBAL_POWER_POLICY", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"user": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "IdleAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "IdleDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "IdleTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "IdleTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "IdleSensitivityAc": SimTypeChar(label="Byte"), "IdleSensitivityDc": SimTypeChar(label="Byte"), "ThrottlePolicyAc": SimTypeChar(label="Byte"), "ThrottlePolicyDc": SimTypeChar(label="Byte"), "MaxSleepAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "MaxSleepDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "Reserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 2), "VideoTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "VideoTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "SpindownTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "SpindownTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "OptimizeForPowerAc": SimTypeChar(label="Byte"), "OptimizeForPowerDc": SimTypeChar(label="Byte"), "FanThrottleToleranceAc": SimTypeChar(label="Byte"), "FanThrottleToleranceDc": SimTypeChar(label="Byte"), "ForcedThrottleAc": SimTypeChar(label="Byte"), "ForcedThrottleDc": SimTypeChar(label="Byte")}, name="USER_POWER_POLICY", pack=False, align=None), "mach": SimStruct({"Revision": SimTypeInt(signed=False, label="UInt32"), "MinSleepAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "MinSleepDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "ReducedLatencySleepAc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "ReducedLatencySleepDc": SimTypeInt(signed=False, label="SYSTEM_POWER_STATE"), "DozeTimeoutAc": SimTypeInt(signed=False, label="UInt32"), "DozeTimeoutDc": SimTypeInt(signed=False, label="UInt32"), "DozeS4TimeoutAc": SimTypeInt(signed=False, label="UInt32"), "DozeS4TimeoutDc": SimTypeInt(signed=False, label="UInt32"), "MinThrottleAc": SimTypeChar(label="Byte"), "MinThrottleDc": SimTypeChar(label="Byte"), "pad1": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 2), "OverThrottledAc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None), "OverThrottledDc": SimStruct({"Action": SimTypeInt(signed=False, label="POWER_ACTION"), "Flags": SimTypeInt(signed=False, label="UInt32"), "EventCode": SimTypeInt(signed=False, label="POWER_ACTION_POLICY_EVENT_CODE")}, name="POWER_ACTION_POLICY", pack=False, align=None)}, name="MACHINE_POWER_POLICY", pack=False, align=None)}, name="POWER_POLICY", pack=False, align=None), offset=0)], SimTypeChar(label="Byte"), arg_names=["pGlobalPowerPolicy", "pPowerPolicy"]),
        # 
        'PowerIsSettingRangeDefined': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeChar(label="Byte"), arg_names=["SubKeyGuid", "SettingGuid"]),
        # 
        'PowerSettingAccessCheckEx': SimTypeFunction([SimTypeInt(signed=False, label="POWER_DATA_ACCESSOR"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="REG_SAM_FLAGS")], SimTypeInt(signed=False, label="UInt32"), arg_names=["AccessFlags", "PowerGuid", "AccessType"]),
        # 
        'PowerSettingAccessCheck': SimTypeFunction([SimTypeInt(signed=False, label="POWER_DATA_ACCESSOR"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["AccessFlags", "PowerGuid"]),
        # 
        'PowerReadACValueIndex': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SchemeGuid", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "AcValueIndex"]),
        # 
        'PowerReadDCValueIndex': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SchemeGuid", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "DcValueIndex"]),
        # 
        'PowerReadFriendlyName': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SchemeGuid", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "Buffer", "BufferSize"]),
        # 
        'PowerReadDescription': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SchemeGuid", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "Buffer", "BufferSize"]),
        # 
        'PowerReadPossibleValue': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "Type", "PossibleSettingIndex", "Buffer", "BufferSize"]),
        # 
        'PowerReadPossibleFriendlyName': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "PossibleSettingIndex", "Buffer", "BufferSize"]),
        # 
        'PowerReadPossibleDescription': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "PossibleSettingIndex", "Buffer", "BufferSize"]),
        # 
        'PowerReadValueMin': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "ValueMinimum"]),
        # 
        'PowerReadValueMax': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "ValueMaximum"]),
        # 
        'PowerReadValueIncrement': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "ValueIncrement"]),
        # 
        'PowerReadValueUnitsSpecifier': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "Buffer", "BufferSize"]),
        # 
        'PowerReadACDefaultIndex': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SchemePersonalityGuid", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "AcDefaultIndex"]),
        # 
        'PowerReadDCDefaultIndex': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SchemePersonalityGuid", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "DcDefaultIndex"]),
        # 
        'PowerReadIconResourceSpecifier': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SchemeGuid", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "Buffer", "BufferSize"]),
        # 
        'PowerReadSettingAttributes': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["SubGroupGuid", "PowerSettingGuid"]),
        # 
        'PowerWriteFriendlyName': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SchemeGuid", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "Buffer", "BufferSize"]),
        # 
        'PowerWriteDescription': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SchemeGuid", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "Buffer", "BufferSize"]),
        # 
        'PowerWritePossibleValue': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "Type", "PossibleSettingIndex", "Buffer", "BufferSize"]),
        # 
        'PowerWritePossibleFriendlyName': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "PossibleSettingIndex", "Buffer", "BufferSize"]),
        # 
        'PowerWritePossibleDescription': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "PossibleSettingIndex", "Buffer", "BufferSize"]),
        # 
        'PowerWriteValueMin': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "ValueMinimum"]),
        # 
        'PowerWriteValueMax': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "ValueMaximum"]),
        # 
        'PowerWriteValueIncrement': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "ValueIncrement"]),
        # 
        'PowerWriteValueUnitsSpecifier': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "Buffer", "BufferSize"]),
        # 
        'PowerWriteACDefaultIndex': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootSystemPowerKey", "SchemePersonalityGuid", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "DefaultAcIndex"]),
        # 
        'PowerWriteDCDefaultIndex': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootSystemPowerKey", "SchemePersonalityGuid", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "DefaultDcIndex"]),
        # 
        'PowerWriteIconResourceSpecifier': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SchemeGuid", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "Buffer", "BufferSize"]),
        # 
        'PowerWriteSettingAttributes': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["SubGroupGuid", "PowerSettingGuid", "Attributes"]),
        # 
        'PowerDuplicateScheme': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Guid"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SourceSchemeGuid", "DestinationSchemeGuid"]),
        # 
        'PowerImportPowerScheme': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Guid"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "ImportFileNamePath", "DestinationSchemeGuid"]),
        # 
        'PowerDeleteScheme': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SchemeGuid"]),
        # 
        'PowerRemovePowerSetting': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["PowerSettingSubKeyGuid", "PowerSettingGuid"]),
        # 
        'PowerCreateSetting': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootSystemPowerKey", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid"]),
        # 
        'PowerCreatePossibleSetting': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootSystemPowerKey", "SubGroupOfPowerSettingsGuid", "PowerSettingGuid", "PossibleSettingIndex"]),
        # 
        'PowerEnumerate': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="POWER_DATA_ACCESSOR"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RootPowerKey", "SchemeGuid", "SubGroupOfPowerSettingsGuid", "AccessFlags", "Index", "Buffer", "BufferSize"]),
        # 
        'PowerOpenUserPowerKey': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["phUserPowerKey", "Access", "OpenExisting"]),
        # 
        'PowerOpenSystemPowerKey': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["phSystemPowerKey", "Access", "OpenExisting"]),
        # 
        'PowerCanRestoreIndividualDefaultPowerScheme': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["SchemeGuid"]),
        # 
        'PowerRestoreIndividualDefaultPowerScheme': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["SchemeGuid"]),
        # 
        'PowerRestoreDefaultPowerSchemes': SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")),
        # 
        'PowerReplaceDefaultPowerSchemes': SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")),
        # 
        'PowerDeterminePlatformRole': SimTypeFunction([], SimTypeInt(signed=False, label="POWER_PLATFORM_ROLE")),
        # 
        'DevicePowerEnumDevices': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeChar(label="Byte"), arg_names=["QueryIndex", "QueryInterpretationFlags", "QueryFlags", "pReturnBuffer", "pBufferSize"]),
        # 
        'DevicePowerSetDeviceState': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["DeviceDescription", "SetFlags", "SetData"]),
        # 
        'DevicePowerOpen': SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["DebugMask"]),
        # 
        'DevicePowerClose': SimTypeFunction([], SimTypeChar(label="Byte")),
        # 
        'PowerReportThermalEvent': SimTypeFunction([SimTypePointer(SimStruct({"Version": SimTypeInt(signed=False, label="UInt32"), "Size": SimTypeInt(signed=False, label="UInt32"), "Type": SimTypeInt(signed=False, label="UInt32"), "Temperature": SimTypeInt(signed=False, label="UInt32"), "TripPointTemperature": SimTypeInt(signed=False, label="UInt32"), "Initiator": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="THERMAL_EVENT", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Event"]),
    }

lib.set_prototypes(prototypes)



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("api-ms-win-shcore-scaling-l1-1-0.dll")
prototypes = \
    {
        # 
        'GetScaleFactorForDevice': SimTypeFunction([SimTypeInt(signed=False, label="DISPLAY_DEVICE_TYPE")], SimTypeInt(signed=False, label="DEVICE_SCALE_FACTOR"), arg_names=["deviceType"]),
        # 
        'RegisterScaleChangeNotifications': SimTypeFunction([SimTypeInt(signed=False, label="DISPLAY_DEVICE_TYPE"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["displayDevice", "hwndNotify", "uMsgNotify", "pdwCookie"]),
        # 
        'RevokeScaleChangeNotifications': SimTypeFunction([SimTypeInt(signed=False, label="DISPLAY_DEVICE_TYPE"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["displayDevice", "dwCookie"]),
    }

lib.set_prototypes(prototypes)



from .ArchPcode_CR16C_LE_16_default import ArchPcode_CR16C_LE_16_default
from .ArchPcode_ARM_LE_32_v8 import ArchPcode_ARM_LE_32_v8
from .ArchPcode_ARM_LE_32_v8T import ArchPcode_ARM_LE_32_v8T
from .ArchPcode_ARM_LEBE_32_v8LEInstruction import ArchPcode_ARM_LEBE_32_v8LEInstruction
from .ArchPcode_ARM_BE_32_v8 import ArchPcode_ARM_BE_32_v8
from .ArchPcode_ARM_BE_32_v8T import ArchPcode_ARM_BE_32_v8T
from .ArchPcode_ARM_LE_32_v7 import ArchPcode_ARM_LE_32_v7
from .ArchPcode_ARM_LEBE_32_v7LEInstruction import ArchPcode_ARM_LEBE_32_v7LEInstruction
from .ArchPcode_ARM_BE_32_v7 import ArchPcode_ARM_BE_32_v7
from .ArchPcode_ARM_LE_32_Cortex import ArchPcode_ARM_LE_32_Cortex
from .ArchPcode_ARM_BE_32_Cortex import ArchPcode_ARM_BE_32_Cortex
from .ArchPcode_ARM_LE_32_v6 import ArchPcode_ARM_LE_32_v6
from .ArchPcode_ARM_BE_32_v6 import ArchPcode_ARM_BE_32_v6
from .ArchPcode_ARM_LE_32_v5t import ArchPcode_ARM_LE_32_v5t
from .ArchPcode_ARM_BE_32_v5t import ArchPcode_ARM_BE_32_v5t
from .ArchPcode_ARM_LE_32_v5 import ArchPcode_ARM_LE_32_v5
from .ArchPcode_ARM_BE_32_v5 import ArchPcode_ARM_BE_32_v5
from .ArchPcode_ARM_LE_32_v4t import ArchPcode_ARM_LE_32_v4t
from .ArchPcode_ARM_BE_32_v4t import ArchPcode_ARM_BE_32_v4t
from .ArchPcode_ARM_LE_32_v4 import ArchPcode_ARM_LE_32_v4
from .ArchPcode_ARM_BE_32_v4 import ArchPcode_ARM_BE_32_v4
from .ArchPcode_8048_LE_16_default import ArchPcode_8048_LE_16_default
from .ArchPcode_PIC_17_LE_16_PIC_17C7xx import ArchPcode_PIC_17_LE_16_PIC_17C7xx
from .ArchPcode_PIC_18_LE_24_PIC_18 import ArchPcode_PIC_18_LE_24_PIC_18
from .ArchPcode_PIC_12_LE_16_PIC_12C5xx import ArchPcode_PIC_12_LE_16_PIC_12C5xx
from .ArchPcode_PIC_16_LE_16_PIC_16C5x import ArchPcode_PIC_16_LE_16_PIC_16C5x
from .ArchPcode_PIC_16_LE_16_PIC_16 import ArchPcode_PIC_16_LE_16_PIC_16
from .ArchPcode_PIC_16_LE_16_PIC_16F import ArchPcode_PIC_16_LE_16_PIC_16F
from .ArchPcode_PIC_24E_LE_24_default import ArchPcode_PIC_24E_LE_24_default
from .ArchPcode_PIC_24F_LE_24_default import ArchPcode_PIC_24F_LE_24_default
from .ArchPcode_PIC_24H_LE_24_default import ArchPcode_PIC_24H_LE_24_default
from .ArchPcode_dsPIC30F_LE_24_default import ArchPcode_dsPIC30F_LE_24_default
from .ArchPcode_dsPIC33F_LE_24_default import ArchPcode_dsPIC33F_LE_24_default
from .ArchPcode_dsPIC33E_LE_24_default import ArchPcode_dsPIC33E_LE_24_default
from .ArchPcode_dsPIC33C_LE_24_default import ArchPcode_dsPIC33C_LE_24_default
from .ArchPcode_PowerPC_BE_32_default import ArchPcode_PowerPC_BE_32_default
from .ArchPcode_PowerPC_LE_32_default import ArchPcode_PowerPC_LE_32_default
from .ArchPcode_PowerPC_BE_64_default import ArchPcode_PowerPC_BE_64_default
from .ArchPcode_PowerPC_BE_64_64_32addr import ArchPcode_PowerPC_BE_64_64_32addr
from .ArchPcode_PowerPC_LE_64_64_32addr import ArchPcode_PowerPC_LE_64_64_32addr
from .ArchPcode_PowerPC_LE_64_default import ArchPcode_PowerPC_LE_64_default
from .ArchPcode_PowerPC_BE_32_4xx import ArchPcode_PowerPC_BE_32_4xx
from .ArchPcode_PowerPC_LE_32_4xx import ArchPcode_PowerPC_LE_32_4xx
from .ArchPcode_PowerPC_BE_32_MPC8270 import ArchPcode_PowerPC_BE_32_MPC8270
from .ArchPcode_PowerPC_BE_32_QUICC import ArchPcode_PowerPC_BE_32_QUICC
from .ArchPcode_PowerPC_LE_32_QUICC import ArchPcode_PowerPC_LE_32_QUICC
from .ArchPcode_PowerPC_BE_64_A2_32addr import ArchPcode_PowerPC_BE_64_A2_32addr
from .ArchPcode_PowerPC_LE_64_A2_32addr import ArchPcode_PowerPC_LE_64_A2_32addr
from .ArchPcode_PowerPC_BE_64_A2ALT_32addr import ArchPcode_PowerPC_BE_64_A2ALT_32addr
from .ArchPcode_PowerPC_LE_64_A2ALT_32addr import ArchPcode_PowerPC_LE_64_A2ALT_32addr
from .ArchPcode_PowerPC_BE_64_A2ALT import ArchPcode_PowerPC_BE_64_A2ALT
from .ArchPcode_PowerPC_LE_64_A2ALT import ArchPcode_PowerPC_LE_64_A2ALT
from .ArchPcode_PowerPC_BE_64_VLE_32addr import ArchPcode_PowerPC_BE_64_VLE_32addr
from .ArchPcode_PowerPC_BE_64_VLEALT_32addr import ArchPcode_PowerPC_BE_64_VLEALT_32addr
from .ArchPcode_MCS96_LE_16_default import ArchPcode_MCS96_LE_16_default
from .ArchPcode_SuperH4_BE_32_default import ArchPcode_SuperH4_BE_32_default
from .ArchPcode_SuperH4_LE_32_default import ArchPcode_SuperH4_LE_32_default
from .ArchPcode_SuperH_BE_32_SH_2A import ArchPcode_SuperH_BE_32_SH_2A
from .ArchPcode_SuperH_BE_32_SH_2 import ArchPcode_SuperH_BE_32_SH_2
from .ArchPcode_SuperH_BE_32_SH_1 import ArchPcode_SuperH_BE_32_SH_1
from .ArchPcode_CP1600_BE_16_default import ArchPcode_CP1600_BE_16_default
from .ArchPcode_TI_MSP430_LE_16_default import ArchPcode_TI_MSP430_LE_16_default
from .ArchPcode_TI_MSP430X_LE_32_default import ArchPcode_TI_MSP430X_LE_32_default
from .ArchPcode_x86_LE_32_default import ArchPcode_x86_LE_32_default
from .ArchPcode_x86_LE_32_System_Management_Mode import ArchPcode_x86_LE_32_System_Management_Mode
from .ArchPcode_x86_LE_16_Real_Mode import ArchPcode_x86_LE_16_Real_Mode
from .ArchPcode_x86_LE_16_Protected_Mode import ArchPcode_x86_LE_16_Protected_Mode
from .ArchPcode_x86_LE_64_default import ArchPcode_x86_LE_64_default
from .ArchPcode_V850_LE_32_default import ArchPcode_V850_LE_32_default
from .ArchPcode_6805_BE_16_default import ArchPcode_6805_BE_16_default
from .ArchPcode_6809_BE_16_default import ArchPcode_6809_BE_16_default
from .ArchPcode_z80_LE_16_default import ArchPcode_z80_LE_16_default
from .ArchPcode_z8401x_LE_16_default import ArchPcode_z8401x_LE_16_default
from .ArchPcode_z180_LE_16_default import ArchPcode_z180_LE_16_default
from .ArchPcode_z182_LE_16_default import ArchPcode_z182_LE_16_default
from .ArchPcode_Toy_BE_32_default import ArchPcode_Toy_BE_32_default
from .ArchPcode_Toy_BE_32_posStack import ArchPcode_Toy_BE_32_posStack
from .ArchPcode_Toy_LE_32_default import ArchPcode_Toy_LE_32_default
from .ArchPcode_Toy_BE_32_wordSize2 import ArchPcode_Toy_BE_32_wordSize2
from .ArchPcode_Toy_LE_32_wordSize2 import ArchPcode_Toy_LE_32_wordSize2
from .ArchPcode_Toy_BE_64_default import ArchPcode_Toy_BE_64_default
from .ArchPcode_Toy_BE_64_harvard import ArchPcode_Toy_BE_64_harvard
from .ArchPcode_Toy_LE_64_default import ArchPcode_Toy_LE_64_default
from .ArchPcode_Toy_BE_32_builder import ArchPcode_Toy_BE_32_builder
from .ArchPcode_Toy_LE_32_builder import ArchPcode_Toy_LE_32_builder
from .ArchPcode_Toy_BE_32_builder_align2 import ArchPcode_Toy_BE_32_builder_align2
from .ArchPcode_Toy_LE_32_builder_align2 import ArchPcode_Toy_LE_32_builder_align2
from .ArchPcode_Dalvik_LE_32_default import ArchPcode_Dalvik_LE_32_default
from .ArchPcode_8051_BE_16_default import ArchPcode_8051_BE_16_default
from .ArchPcode_80251_BE_24_default import ArchPcode_80251_BE_24_default
from .ArchPcode_80390_BE_24_default import ArchPcode_80390_BE_24_default
from .ArchPcode_8051_BE_24_mx51 import ArchPcode_8051_BE_24_mx51
from .ArchPcode_pa_risc_BE_32_default import ArchPcode_pa_risc_BE_32_default
from .ArchPcode_M8C_BE_16_default import ArchPcode_M8C_BE_16_default
from .ArchPcode_RISCV_LE_64_RV64I import ArchPcode_RISCV_LE_64_RV64I
from .ArchPcode_RISCV_LE_64_RV64IC import ArchPcode_RISCV_LE_64_RV64IC
from .ArchPcode_RISCV_LE_64_RV64G import ArchPcode_RISCV_LE_64_RV64G
from .ArchPcode_RISCV_LE_64_RV64GC import ArchPcode_RISCV_LE_64_RV64GC
from .ArchPcode_RISCV_LE_64_default import ArchPcode_RISCV_LE_64_default
from .ArchPcode_RISCV_LE_32_RV32I import ArchPcode_RISCV_LE_32_RV32I
from .ArchPcode_RISCV_LE_32_RV32IC import ArchPcode_RISCV_LE_32_RV32IC
from .ArchPcode_RISCV_LE_32_RV32IMC import ArchPcode_RISCV_LE_32_RV32IMC
from .ArchPcode_RISCV_LE_32_RV32G import ArchPcode_RISCV_LE_32_RV32G
from .ArchPcode_RISCV_LE_32_RV32GC import ArchPcode_RISCV_LE_32_RV32GC
from .ArchPcode_RISCV_LE_32_default import ArchPcode_RISCV_LE_32_default
from .ArchPcode_8085_LE_16_default import ArchPcode_8085_LE_16_default
from .ArchPcode_tricore_LE_32_default import ArchPcode_tricore_LE_32_default
from .ArchPcode_tricore_LE_32_tc29x import ArchPcode_tricore_LE_32_tc29x
from .ArchPcode_tricore_LE_32_tc172x import ArchPcode_tricore_LE_32_tc172x
from .ArchPcode_tricore_LE_32_tc176x import ArchPcode_tricore_LE_32_tc176x
from .ArchPcode_MIPS_BE_32_default import ArchPcode_MIPS_BE_32_default
from .ArchPcode_MIPS_LE_32_default import ArchPcode_MIPS_LE_32_default
from .ArchPcode_MIPS_BE_32_R6 import ArchPcode_MIPS_BE_32_R6
from .ArchPcode_MIPS_LE_32_R6 import ArchPcode_MIPS_LE_32_R6
from .ArchPcode_MIPS_BE_64_default import ArchPcode_MIPS_BE_64_default
from .ArchPcode_MIPS_LE_64_default import ArchPcode_MIPS_LE_64_default
from .ArchPcode_MIPS_BE_64_micro import ArchPcode_MIPS_BE_64_micro
from .ArchPcode_MIPS_LE_64_micro import ArchPcode_MIPS_LE_64_micro
from .ArchPcode_MIPS_BE_64_R6 import ArchPcode_MIPS_BE_64_R6
from .ArchPcode_MIPS_LE_64_R6 import ArchPcode_MIPS_LE_64_R6
from .ArchPcode_MIPS_BE_64_64_32addr import ArchPcode_MIPS_BE_64_64_32addr
from .ArchPcode_MIPS_LE_64_64_32addr import ArchPcode_MIPS_LE_64_64_32addr
from .ArchPcode_MIPS_LE_64_micro64_32addr import ArchPcode_MIPS_LE_64_micro64_32addr
from .ArchPcode_MIPS_BE_64_micro64_32addr import ArchPcode_MIPS_BE_64_micro64_32addr
from .ArchPcode_MIPS_BE_64_64_32R6addr import ArchPcode_MIPS_BE_64_64_32R6addr
from .ArchPcode_MIPS_LE_64_64_32R6addr import ArchPcode_MIPS_LE_64_64_32R6addr
from .ArchPcode_MIPS_BE_32_micro import ArchPcode_MIPS_BE_32_micro
from .ArchPcode_MIPS_LE_32_micro import ArchPcode_MIPS_LE_32_micro
from .ArchPcode_HCS08_BE_16_default import ArchPcode_HCS08_BE_16_default
from .ArchPcode_HCS08_BE_16_MC9S08GB60 import ArchPcode_HCS08_BE_16_MC9S08GB60
from .ArchPcode_HC08_BE_16_default import ArchPcode_HC08_BE_16_default
from .ArchPcode_HC08_BE_16_MC68HC908QY4 import ArchPcode_HC08_BE_16_MC68HC908QY4
from .ArchPcode_HC05_BE_16_default import ArchPcode_HC05_BE_16_default
from .ArchPcode_HC05_BE_16_M68HC05TB import ArchPcode_HC05_BE_16_M68HC05TB
from .ArchPcode_sparc_BE_32_default import ArchPcode_sparc_BE_32_default
from .ArchPcode_sparc_BE_64_default import ArchPcode_sparc_BE_64_default
from .ArchPcode_avr32_BE_32_default import ArchPcode_avr32_BE_32_default
from .ArchPcode_avr8_LE_16_default import ArchPcode_avr8_LE_16_default
from .ArchPcode_avr8_LE_16_extended import ArchPcode_avr8_LE_16_extended
from .ArchPcode_avr8_LE_16_atmega256 import ArchPcode_avr8_LE_16_atmega256
from .ArchPcode_avr8_LE_24_xmega import ArchPcode_avr8_LE_24_xmega
from .ArchPcode_AARCH64_LE_64_v8A import ArchPcode_AARCH64_LE_64_v8A
from .ArchPcode_AARCH64_BE_64_v8A import ArchPcode_AARCH64_BE_64_v8A
from .ArchPcode_6502_LE_16_default import ArchPcode_6502_LE_16_default
from .ArchPcode_6502_BE_16_default import ArchPcode_6502_BE_16_default
from .ArchPcode_HCS12_BE_24_default import ArchPcode_HCS12_BE_24_default
from .ArchPcode_JVM_BE_32_default import ArchPcode_JVM_BE_32_default
from .ArchPcode_DATA_LE_64_default import ArchPcode_DATA_LE_64_default
from .ArchPcode_DATA_BE_64_default import ArchPcode_DATA_BE_64_default
from .ArchPcode_68000_BE_32_default import ArchPcode_68000_BE_32_default
from .ArchPcode_68000_BE_32_MC68030 import ArchPcode_68000_BE_32_MC68030
from .ArchPcode_68000_BE_32_MC68020 import ArchPcode_68000_BE_32_MC68020
from .ArchPcode_68000_BE_32_Coldfire import ArchPcode_68000_BE_32_Coldfire
from .common import *



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("ualapi.dll")
prototypes = \
    {
        # 
        'UalStart': SimTypeFunction([SimTypePointer(SimStruct({"Size": SimTypeInt(signed=False, label="UInt32"), "RoleGuid": SimTypeBottom(label="Guid"), "TenantId": SimTypeBottom(label="Guid"), "Address": SimStruct({"ss_family": SimTypeShort(signed=False, label="UInt16"), "__ss_pad1": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 6), "__ss_align": SimTypeLongLong(signed=True, label="Int64"), "__ss_pad2": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 112)}, name="SOCKADDR_STORAGE", pack=False, align=None), "UserName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260)}, name="UAL_DATA_BLOB", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Data"]),
        # 
        'UalStop': SimTypeFunction([SimTypePointer(SimStruct({"Size": SimTypeInt(signed=False, label="UInt32"), "RoleGuid": SimTypeBottom(label="Guid"), "TenantId": SimTypeBottom(label="Guid"), "Address": SimStruct({"ss_family": SimTypeShort(signed=False, label="UInt16"), "__ss_pad1": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 6), "__ss_align": SimTypeLongLong(signed=True, label="Int64"), "__ss_pad2": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 112)}, name="SOCKADDR_STORAGE", pack=False, align=None), "UserName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260)}, name="UAL_DATA_BLOB", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Data"]),
        # 
        'UalInstrument': SimTypeFunction([SimTypePointer(SimStruct({"Size": SimTypeInt(signed=False, label="UInt32"), "RoleGuid": SimTypeBottom(label="Guid"), "TenantId": SimTypeBottom(label="Guid"), "Address": SimStruct({"ss_family": SimTypeShort(signed=False, label="UInt16"), "__ss_pad1": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 6), "__ss_align": SimTypeLongLong(signed=True, label="Int64"), "__ss_pad2": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 112)}, name="SOCKADDR_STORAGE", pack=False, align=None), "UserName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260)}, name="UAL_DATA_BLOB", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Data"]),
        # 
        'UalRegisterProduct': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["wszProductName", "wszRoleName", "wszGuid"]),
    }

lib.set_prototypes(prototypes)



# This module contains data structures for handling memory, code, and register references.

import logging
l = logging.getLogger(name=__name__)

_noneset = frozenset()

from .sim_event import SimEvent


class SimAction(SimEvent):
    """
    A SimAction represents a semantic action that an analyzed program performs.
    """
    #__slots__ = [ 'bbl_addr', 'inst_addr', 'stmt_idx' ]

    TMP = 'tmp'
    REG = 'reg'
    MEM = 'mem'
    _MAX_ACTION_ID = -1

    def __init__(self, state, region_type):
        """
        Initializes the SimAction.

        :param state: the state that's the SimAction is taking place in.
        """
        SimEvent.__init__(self, state, 'action')
        self.type = region_type
        SimAction._MAX_ACTION_ID += 1
        self._action_id = SimAction._MAX_ACTION_ID

    def __repr__(self):
        if self.sim_procedure is not None:
            location = "%s()" % self.sim_procedure.display_name
        else:
            if self.stmt_idx is not None:
                location = "0x%x:%d" % (self.bbl_addr, self.stmt_idx)
            else:
                location = "0x%x" % self.bbl_addr
        return "<%s %s %s>" % (self.__class__.__name__, location, self._desc())

    def _desc(self):
        raise NotImplementedError()

    #def __getstate__(self):
    #   return { k: getattr(self, k) for k in sum([ c.__slots__ for c in self.__class__.mro() if hasattr(c, '__slots__')], []) } #pylint:disable=no-member
    #def __setstate__(self, s):
    #   for k,v in s.items():
    #       setattr(self, k, v)

    @staticmethod
    def _make_object(v):
        if v is None:
            return None
        elif isinstance(v, SimActionObject):
            return v
        else:
            return SimActionObject(v, reg_deps=None, tmp_deps=None)

    @staticmethod
    def _copy_object(v):
        if isinstance(v, SimActionObject):
            return v.copy()
        else:
            return None

    @property
    def all_objects(self):
        raise NotImplementedError()

    @property
    def is_symbolic(self):
        raise NotImplementedError()

    @property
    def tmp_deps(self):
        return frozenset.union(*[v.tmp_deps for v in self.all_objects])

    @property
    def reg_deps(self):
        return frozenset.union(*[v.reg_deps for v in self.all_objects])

    def _copy_objects(self, c):
        raise NotImplementedError()

    def copy(self):
        c = self._copy_event()
        self._copy_objects(c)

        return c

    def downsize(self):
        """
        Clears some low-level details (that take up memory) out of the SimAction.
        """
        pass


class SimActionExit(SimAction):
    """
    An Exit action represents a (possibly conditional) jump.
    """

    CONDITIONAL = 'conditional'
    DEFAULT = 'default'

    def __init__(self, state, target, condition=None, exit_type=None):
        super().__init__(state, "exit")
        if exit_type is not None:
            self.exit_type = exit_type
        elif condition is None:
            self.exit_type = SimActionExit.CONDITIONAL
        else:
            self.exit_type = SimActionExit.DEFAULT

        self.target = self._make_object(target)
        self.condition = self._make_object(condition)

    def _desc(self):
        return self.exit_type

    @property
    def all_objects(self):
        return [ a for a in ( self.target, self.condition ) if a is not None ]

    @property
    def is_symbolic(self):
        return getattr(self.target, "symbolic", False)

    def _copy_objects(self, c):
        c.exit_type = self.exit_type
        c.target = self._copy_object(self.target)
        c.condition = self._copy_object(self.condition)


class SimActionConstraint(SimAction):
    """
    A constraint action represents an extra constraint added during execution of a path.
    """

    def __init__(self, state, constraint, condition=None):
        super().__init__(state, "constraint")

        self.constraint = self._make_object(constraint)
        self.condition = self._make_object(condition)

    @property
    def all_objects(self):
        return [ a for a in ( self.constraint, self.condition ) if a is not None ]

    @property
    def is_symbolic(self):
        return getattr(self.constraint, "symbolic", False)

    def _copy_objects(self, c):
        c.constraint = self._copy_object(self.constraint)
        c.condition = self._copy_object(self.condition)

    def _desc(self):
        s = '%s' % str(self.constraint)
        if self.condition is not None:
            s += ' (cond)'
        return s


class SimActionOperation(SimAction):
    """
    An action representing an operation between variables and/or constants.
    """

    def __init__(self, state, op, exprs, result):
        super().__init__(state, 'operation')

        self.op = op
        self.exprs = exprs

        self.result = result

    @property
    def all_objects(self):
        return [ ex for ex in self.exprs if isinstance(ex, SimActionObject) ]

    @property
    def is_symbolic(self):
        return any(getattr(ex, "symbolic", False) for ex in self.exprs)

    def _copy_objects(self, c):
        c.op = self.op
        c.exprs = self.exprs[::]
        c.result = self.result

    def _desc(self):
        return "operation/%s" % (self.op)


class SimActionData(SimAction):
    """
    A Data action represents a read or a write from memory, registers or a file.
    """
    #__slots__ = [ 'objects' ]

    READ = 'read'
    WRITE = 'write'
    OPERATE = 'operate'

    def __init__(self, state, region_type, action, tmp=None, addr=None, size=None, data=None, condition=None,
                 fallback=None, fd=None):
        super().__init__(state, region_type)
        self.action = action

        self._reg_dep = _noneset if addr is None or action != SimActionData.READ or not isinstance(addr, int) else frozenset((addr,))
        self._tmp_dep = _noneset if tmp is None or action != SimActionData.READ else frozenset((tmp,))

        self.tmp = tmp
        self.offset = None
        if region_type == 'reg':
            if isinstance(addr, int):
                self.offset = addr
            else:
                if addr.symbolic:
                    # FIXME: we should fix it by allowing .offset taking ASTs instead of concretizing it right away
                    l.warning('Concretizing a symbolic register offset in SimActionData.')
                    self.offset = state.solver.eval(addr)
                else:
                    # it's not symbolic
                    self.offset = state.solver.eval_one(addr)
        self.addr = self._make_object(addr)
        self.size = self._make_object(size)
        self.data = self._make_object(data)
        self.condition = self._make_object(condition)
        self.fallback = self._make_object(fallback)
        self.fd = self._make_object(fd)

        # these are extra attributes that expose low-level effects, such as the *actual*
        # written value
        self.actual_addrs = None
        # `actual_value` always stores whatever the data looks like in memory from left to right, therefore it's always
        # big-endian (if endianness matters)
        self.actual_value = None
        self.added_constraints = None

    def downsize(self):
        self.actual_addrs = None
        self.actual_value = None
        self.added_constraints = None

    @property
    def all_objects(self):
        return [ a for a in [ self.addr, self.size, self.data, self.condition, self.fallback, self.fd ] if a is not None ]

    @property
    def is_symbolic(self):
        return any(getattr(a, "symbolic", False) for a in [ self.addr, self.size, self.data ] if a is not None)

    @property
    def tmp_deps(self):
        return super().tmp_deps | self._tmp_dep

    @property
    def reg_deps(self):
        return super().reg_deps | self._reg_dep

    def _desc(self):
        def _repr(o):
            if type(o) in {bytes, str, int}:
                return o
            try:
                o = o.ast
            except AttributeError:
                pass
            if type(o) in {bytes, str, int}:
                return o
            return o.shallow_repr()
        if self.type == 'reg':
            _size = self.size.ast if isinstance(self.size, SimActionObject) else self.size
            assert isinstance(_size, int)
            storage = self.arch.register_size_names[(self.offset, _size // self.arch.byte_width)]
        elif self.type == 'tmp':
            storage = f'tmp_{self.tmp}'
        else:
            storage = self.addr
        direction = '<<----' if self.action == 'write' else '---->>'
        return f"{self.type}/{self.action}: {_repr(storage)}  {direction}  {_repr(self.data)}"

    def _copy_objects(self, c):
        c.action = self.action
        c.tmp = self.tmp
        c.addr = self._copy_object(self.addr)
        c.size = self._copy_object(self.size)
        c.data = self._copy_object(self.data)
        c.condition = self._copy_object(self.condition)
        c.fallback = self._copy_object(self.fallback)
        c.fd = self._copy_object(self.fd)

from .sim_action_object import SimActionObject
